part of masamune.flutter;

/// NavigatorState extension methods.
extension NavigatorStateExtension on NavigatorState {
  /// Replace page.
  /// 
  /// [path]: Page path.
  /// [document]: Document data to pass to the next page.
  /// [documentPath]: Document data path to pass to the next page.
  /// [routePath]: Page root path.
  /// [fullscreenDialog]: Modal display.
  /// [pageDataPath]: Path to get passed data.
  Future replacePath(
    String path,
    {
      IDataDocument document,
      String documentPath,
      String routePath = DefaultPath.route,
      String pageDataPath = DefaultPath.pageData,
      bool fullscreenDialog = false
    }
  ) async {
    if( isEmpty(path) ) return null;
    path = path.trimString(Const.slash);
    DataField data = PathMap.get(Paths.child(routePath, path));
    if( data == null || !( data.data is WidgetBuilder ) ) return null;
    return this.pushReplacement(
      MaterialPageRoute(builder: data.data, fullscreenDialog: fullscreenDialog)
    );
  }
  /// Transition the page by specifying the path.
  /// 
  /// [path]: Page path.
  /// [document]: Document data to pass to the next page.
  /// [documentPath]: Document data path to pass to the next page.
  /// [routePath]: Page root path.
  /// [pageDataPath]: Path to get passed data.
  /// [fullscreenDialog]: Modal display.
  /// [direction]: Page animation report.
  Future pushPath(
    String path,
    {
      IDataDocument document,
      String documentPath,
      String routePath = DefaultPath.route,
      String pageDataPath = DefaultPath.pageData,
      bool fullscreenDialog = false,
      SlideDirection direction
    }
  ) async {
    if( isEmpty(path) ) return null;
    path = path.trimString(Const.slash);
    DataField data = PathMap.get(Paths.child(routePath, path));
    if( data == null || !( data.data is WidgetBuilder ) ) return null;
    return this.push(
      SlidePageRoute(
        path: path,
        child: ( data.data as WidgetBuilder )( this.context ),
        document: document,
        documentPath: documentPath,
        pageDataPath: pageDataPath,
        fullscreenDialog: fullscreenDialog,
        direction: direction
      )
    );
  }
  /// Pops the page to the page with the specified path.
  /// 
  /// [path]: Page path.
  Future popUntilPath( String path ) async {
    if( isEmpty(path) ) return null;
    path = path.trimString(Const.slash);
    this.popUntil(
      (route){
        if( !( route is SlidePageRoute ) ) return true;
        return ( route.asType<SlidePageRoute>().path == path );
      }
    );
  }

}