part of masamune.flutter;

/// Page route to perform slide animation.
class SlidePageRoute extends PageRouteBuilder {
  /// Route path.
  final String path;
  /// Page route to perform slide animation.
  /// 
  /// [path]: Route path.
  /// [child]: Child widget.
  /// [document]: Document data to pass to the next page.
  /// [documentPath]: Document data path to pass to the next page.
  /// [routePath]: Page root path.
  /// [pageDataPath]: Path to get passed data.
  /// [fullscreenDialog]: Modal display.
  /// [direction]: Animation direction.
  SlidePageRoute(
    {
      this.path,
      Widget child,
      IDataDocument document,
      String documentPath,
      String pageDataPath = DefaultPath.pageData,
      bool fullscreenDialog = false,
      SlideDirection direction
    }
  ) : super(
    settings: RouteSettings( name: path, arguments: document ?? PathMap.get( documentPath ) ),
    pageBuilder: ( context, animation, secondaryAnimation ) {
      if( isEmpty( pageDataPath ) ) return child;
      if( document == null && isNotEmpty(documentPath) )
        document = PathMap.get( documentPath );
      if( document != null ){
        IDataDocument dist = PathMap.get<IDataDocument>( pageDataPath );
        if( dist == null ) dist = DataDocument( pageDataPath );
        dist.clear();
        for( MapEntry<String, IDataField> tmp in document.entries ){
          dist[tmp.key] = tmp.value.data;
        }
      }
      return child;
    },
    fullscreenDialog: fullscreenDialog,
    transitionsBuilder: ( context,  animation,  secondaryAnimation,  child ) {
      if( direction == null ){
        if( fullscreenDialog ){
          return SlideTransition(
            position: Tween<Offset>(
              begin: const Offset(0, 1),
              end: Offset.zero,
            ).animate(animation),
            child: child
          );
        }
        direction = SlideDirection.left;
      }
      switch( direction ){
        case SlideDirection.right:
        return SlideTransition(
          position: Tween<Offset>(
            begin: Offset.zero,
            end: const Offset(1, 0),
          ).animate(animation),
          child: child
        );
        case SlideDirection.up:
        return SlideTransition(
          position: Tween<Offset>(
            begin: const Offset(0, 1),
            end: Offset.zero,
          ).animate(animation),
          child: child
        );
        case SlideDirection.down:
        return SlideTransition(
          position: Tween<Offset>(
            begin: Offset.zero,
            end: const Offset(0, 1),
          ).animate(animation),
          child: child
        );
        default:
        return SlideTransition(
          position: Tween<Offset>(
            begin: const Offset(1, 0),
            end: Offset.zero,
          ).animate(animation),
          child: child
        );
      }
    },
  );
}
/// Defines the slide direction.
enum SlideDirection {
  /// Left.
  left,
  /// Right.
  right,
  /// Up.
  up,
  /// Down.
  down
}